from django.contrib.auth.models import AbstractUser
from django.db import models

from simple_email_confirmation.models import SimpleEmailConfirmationUserMixin


class User(SimpleEmailConfirmationUserMixin, AbstractUser):
    pass


class UserWithoutMixin(models.Model):
       email = models.EmailField()



from __future__ import unicode_literals

from django.conf import settings
from django.contrib.auth import get_user_model
from django.db import models
from django.db.models.signals import post_save
from django.utils.crypto import get_random_string
from django.utils import timezone
from django.utils.translation import ugettext_lazy as _

from .exceptions import (
    EmailConfirmationExpired, EmailIsPrimary, EmailNotConfirmed,
)
from .signals import (
    email_confirmed, unconfirmed_email_created, primary_email_changed,
)


class SimpleEmailConfirmationUserMixin(object):
    """
    Mixin to be used with your django 1.5+ custom User model.
    Provides python-level functionality only.
    """

    # if your User object stores the User's primary email address
    # in a place other than User.email, you can override the
    # primary_email_field_name and/or primary_email get/set methods.
    # All access to a User's primary_email in this app passes through
    # these two get/set methods.

    primary_email_field_name = 'email'

    def get_primary_email(self):
        return getattr(self, self.primary_email_field_name)

    def set_primary_email(self, email, require_confirmed=True):
        "Set an email address as primary"
        old_email = self.get_primary_email()
        if email == old_email:
            return

        if email not in self.confirmed_emails and require_confirmed:
            raise EmailNotConfirmed()

        setattr(self, self.primary_email_field_name, email)
        self.save(update_fields=[self.primary_email_field_name])
        primary_email_changed.send(
            sender=self, old_email=old_email, new_email=email,
        )

    @property
    def is_confirmed(self):
        "Is the User's primary email address confirmed?"
        return self.get_primary_email() in self.confirmed_emails

    @property
    def confirmed_at(self):
        "When the User's primary email address was confirmed, or None"
        address = self.email_address_set.get(email=self.get_primary_email())
        return address.confirmed_at

    @property
    def confirmation_key(self):
        """
        Confirmation key for the User's primary email

        DEPRECATED. Use get_confirmation_key() instead.
        """
        email = self.get_primary_email()
        return self.get_confirmation_key(email)

    @property
    def confirmed_emails(self):
        "DEPRECATED. Use get_confirmed_emails() instead."
        return self.get_confirmed_emails()

    @property
    def unconfirmed_emails(self):
        "DEPRECATED. Use get_unconfirmed_emails() instead."
        return self.get_unconfirmed_emails()

    def get_confirmation_key(self, email=None):
        "Get the confirmation key for an email"
        email = email or self.get_primary_email()
        address = self.email_address_set.get(email=email)
        return address.key

    def get_confirmed_emails(self):
        "List of emails this User has confirmed"
        address_qs = self.email_address_set.filter(confirmed_at__isnull=False)
        return [address.email for address in address_qs]

    def get_unconfirmed_emails(self):
        "List of emails this User has been associated with but not confirmed"
        address_qs = self.email_address_set.filter(confirmed_at__isnull=True)
        return [address.email for address in address_qs]

    def confirm_email(self, confirmation_key, save=True):
        """
        Attempt to confirm an email using the given key.
        Returns the email that was confirmed, or raise an exception.
        """
        address = self.email_address_set.confirm(confirmation_key, save=save)
        return address.email

    def add_confirmed_email(self, email):
        "Adds an email to the user that's already in the confirmed state"
        # if email already exists, let exception be thrown
        address = self.email_address_set.create_confirmed(email)
        return address.key

    def add_unconfirmed_email(self, email):
        "Adds an unconfirmed email address and returns it's confirmation key"
        # if email already exists, let exception be thrown
        address = self.email_address_set.create_unconfirmed(email)
        return address.key

    def add_email_if_not_exists(self, email):
        """
        If the user already has the email, and it's confirmed, do nothing
        and return None.

        If the user already has the email, and it's unconfirmed, reset the
        confirmation. If the confirmation is unexpired, do nothing. Return
        the confirmation key of the email.
        """
        try:
            address = self.email_address_set.get(email=email)
        except EmailAddress.DoesNotExist:
            key = self.add_unconfirmed_email(email)
        else:
            if not address.is_confirmed:
                key = address.reset_confirmation()
            else:
                key = None

        return key

    def reset_email_confirmation(self, email):
        "Reset the expiration of an email confirmation"
        address = self.email_address_set.get(email=email)
        return address.reset_confirmation()

    def remove_email(self, email):
        "Remove an email address"
        # if email already exists, let exception be thrown
        if email == self.get_primary_email():
            raise EmailIsPrimary()
        address = self.email_address_set.get(email=email)
        address.delete()


class EmailAddressManager(models.Manager):

    def generate_key(self):
        "Generate a new random key and return it"
        # By default, a length of keys is 12. If you want to change it, set
        # settings.SIMPLE_EMAIL_CONFIRMATION_KEY_LENGTH to integer value (max 40).
        return get_random_string(
            length=min(getattr(settings, 'SIMPLE_EMAIL_CONFIRMATION_KEY_LENGTH', 12), 40)
        )

    def create_confirmed(self, email, user=None):
        "Create an email address in the confirmed state"
        user = user or getattr(self, 'instance', None)
        if not user:
            raise ValueError('Must specify user or call from related manager')
        key = self.generate_key()
        now = timezone.now()
        # let email-already-exists exception propogate through
        address = self.create(
            user=user, email=email, key=key, set_at=now, confirmed_at=now,
        )
        return address

    def create_unconfirmed(self, email, user=None):
        "Create an email address in the unconfirmed state"
        user = user or getattr(self, 'instance', None)
        if not user:
            raise ValueError('Must specify user or call from related manager')
        key = self.generate_key()
        # let email-already-exists exception propogate through
        address = self.create(user=user, email=email, key=key)
        unconfirmed_email_created.send(sender=user, email=email)
        return address

    def confirm(self, key, user=None, save=True):
        "Confirm an email address. Returns the address that was confirmed."
        queryset = self.all()
        if user:
            queryset = queryset.filter(user=user)
        address = queryset.get(key=key)

        if address.is_key_expired:
            raise EmailConfirmationExpired()

        if not address.is_confirmed:
            address.confirmed_at = timezone.now()
            if save:
                address.save(update_fields=['confirmed_at'])
                email_confirmed.send(sender=address.user, email=address.email)

        return address


def get_user_primary_email(user):
    # softly failing on using these methods on `user` to support
    # not using the SimpleEmailConfirmationMixin in your User model
    # https://github.com/mfogel/django-simple-email-confirmation/pull/3
    if hasattr(user, 'get_primary_email'):
        return user.get_primary_email()
    return user.email


class EmailAddress(models.Model):
    "An email address belonging to a User"

    user = models.ForeignKey(
        settings.AUTH_USER_MODEL, related_name='email_address_set',
        on_delete=models.CASCADE,
    )
    email = models.EmailField(max_length=255)
    key = models.CharField(max_length=40, unique=True)

    set_at = models.DateTimeField(
        default=timezone.now,
        help_text=_('When the confirmation key expiration was set'),
    )
    confirmed_at = models.DateTimeField(
        blank=True, null=True,
        help_text=_('First time this email was confirmed'),
    )

    objects = EmailAddressManager()

    class Meta:
        unique_together = (('user', 'email'),)
        verbose_name_plural = "email addresses"

    def __unicode__(self):
        return '{} <{}>'.format(self.user, self.email)

    @property
    def is_confirmed(self):
        return self.confirmed_at is not None

    @property
    def is_primary(self):
        primary_email = get_user_primary_email(self.user)
        return bool(primary_email == self.email)

    @property
    def key_expires_at(self):
        # By default, keys don't expire. If you want them to, set
        # settings.SIMPLE_EMAIL_CONFIRMATION_PERIOD to a timedelta.
        period = getattr(
            settings, 'SIMPLE_EMAIL_CONFIRMATION_PERIOD', None
        )
        return self.set_at + period if period is not None else None

    @property
    def is_key_expired(self):
        return self.key_expires_at and timezone.now() >= self.key_expires_at

    def reset_confirmation(self):
        """
        Re-generate the confirmation key and key expiration associated
        with this email.  Note that the previou confirmation key will
        cease to work.
        """
        self.key = EmailAddress._default_manager.generate_key()
        self.set_at = timezone.now()

        self.confirmed_at = None
        self.save(update_fields=['key', 'set_at', 'confirmed_at'])
        return self.key


# by default, auto-add unconfirmed EmailAddress objects for new Users
if getattr(settings, 'SIMPLE_EMAIL_CONFIRMATION_AUTO_ADD', True):
    def auto_add(sender, **kwargs):
        if sender == get_user_model() and kwargs['created'] and not kwargs['raw']:
            user = kwargs.get('instance')
            email = get_user_primary_email(user)
            if email:
                if hasattr(user, 'add_unconfirmed_email'):
                    user.add_unconfirmed_email(email)
                else:
                    user.email_address_set.create_unconfirmed(email)

    # TODO: try to only connect this to the User model. We can't use
    #       get_user_model() here - results in import loop.

    post_save.connect(auto_add)



from datetime import timedelta
from time import sleep

from django.apps import apps
from django.conf import settings
from django.contrib.auth import get_user_model
from django.test import TestCase
from django.test.utils import override_settings

from ..exceptions import (
    EmailConfirmationExpired, EmailIsPrimary, EmailNotConfirmed,
)
from ..models import EmailAddress, get_user_primary_email
from ..signals import (
    email_confirmed, unconfirmed_email_created, primary_email_changed,
)


class EmailConfirmationTestCase(TestCase):

    def setUp(self):
        email = 'nobody@important.com'
        self.user = get_user_model().objects.create_user('uname', email=email)

    def test_key_generation(self):
        "Generate a few keys and make sure they're unique"
        generator = EmailAddress.objects.generate_key
        key1 = generator()
        key2 = generator()
        key3 = generator()

        self.assertNotEqual(key1, key2)
        self.assertNotEqual(key2, key3)
        self.assertNotEqual(key1, key3)

    def test_key_length(self):
        "Generate a few keys and compare them length with the settings"
        generator = EmailAddress.objects.generate_key
        for _ in range(3):
            key = generator()
            self.assertEqual(len(key), settings.SIMPLE_EMAIL_CONFIRMATION_KEY_LENGTH)

    def test_create_confirmed(self):
        "Add an unconfirmed email for a User"
        email = 'test@test.test'

        key = self.user.add_confirmed_email(email)

        address = self.user.email_address_set.get(email=email)
        self.assertTrue(address.is_confirmed)
        self.assertEqual(address.key, key)

    def test_error_create_no_user(self):
        email = 'test@test.test'
        with self.assertRaises(ValueError):
            EmailAddress.objects.create_confirmed(email)
        with self.assertRaises(ValueError):
            EmailAddress.objects.create_unconfirmed(email)

    def test_create_unconfirmed(self):
        "Add an unconfirmed email for a User"
        email = 'test@test.test'

        # assert signal fires as expected
        def listener(sender, **kwargs):
            self.assertEqual(sender, self.user)
            self.assertEqual(email, kwargs.get('email'))
        unconfirmed_email_created.connect(listener)

        key = self.user.add_unconfirmed_email(email)

        address = self.user.email_address_set.get(email=email)
        self.assertFalse(address.is_confirmed)
        self.assertEqual(address.confirmed_at, None)
        self.assertEqual(address.key, key)

    def test_reset_confirmation(self):
        "Reset a confirmation key"
        email = 'test@test.test'
        self.user.add_unconfirmed_email(email)
        address = self.user.email_address_set.get(email=email)
        org_key, org_at = address.key, address.set_at
        sleep(0.1)
        self.user.reset_email_confirmation(email)

        address = self.user.email_address_set.get(email=email)
        self.assertNotEqual(address.key, org_key)
        self.assertGreater(address.set_at, org_at)

    def test_confirm_email(self):
        "Confirm an outstanding confirmation"
        email1, email2, email3 = '1@t.t', '2@t.t', '3@t.t'
        self.user.add_unconfirmed_email(email1)
        self.user.add_unconfirmed_email(email2)
        self.user.add_unconfirmed_email(email3)

        # assert signal fires as expected
        def listener(sender, **kwargs):
            self.assertEqual(sender, self.user)
            self.assertEqual(kwargs.get('email'), self.user.email)
        email_confirmed.connect(listener)

        self.user.confirm_email(self.user.get_confirmation_key())

        self.assertTrue(self.user.is_confirmed)
        self.assertTrue(self.user.confirmed_at)
        self.assertIn(self.user.email, self.user.get_confirmed_emails())
        self.assertNotIn(self.user.email, self.user.get_unconfirmed_emails())

        self.assertNotIn(email1, self.user.get_confirmed_emails())
        self.assertNotIn(email2, self.user.get_confirmed_emails())
        self.assertNotIn(email3, self.user.get_confirmed_emails())
        self.assertIn(email1, self.user.get_unconfirmed_emails())
        self.assertIn(email2, self.user.get_unconfirmed_emails())
        self.assertIn(email3, self.user.get_unconfirmed_emails())

    def test_confirm_previously_confirmed_confirmation(self):
        "Re-confirm an confirmation that was already confirmed"
        email = 't@t.t'
        key = self.user.add_confirmed_email(email)
        at_before = self.user.email_address_set.get(email=email).confirmed_at

        self.user.confirm_email(key)
        at_after = self.user.email_address_set.get(email=email).confirmed_at

        self.assertIn(email, self.user.get_confirmed_emails())
        self.assertEqual(at_after, at_before)

    @override_settings(SIMPLE_EMAIL_CONFIRMATION_PERIOD=timedelta(weeks=1))
    def test_attempt_confirm_expired_confirmation(self):
        "Try to confirm an expired confirmation"
        email = 't@t.t'
        self.user.add_unconfirmed_email(email)
        address = self.user.email_address_set.get(email=email)
        period = settings.SIMPLE_EMAIL_CONFIRMATION_PERIOD
        address.set_at = address.set_at - period * 2
        address.save()

        with self.assertRaises(EmailConfirmationExpired):
            self.user.confirm_email(address.key)

    def test_attempt_confirm_invalid_key(self):
        "Try to confirm an with an invalid confirmation key"
        email1, email2 = '1@t.t', '2@t.t'
        self.user.add_unconfirmed_email(email1)
        self.user.add_unconfirmed_email(email2)

        invalid_key = 'thisisnotgoingtoappearrandomaly'
        with self.assertRaises(EmailAddress.DoesNotExist):
            self.user.confirm_email(invalid_key)

    def test_remove_email(self):
        email_unconfirmed = 'unconfirmed@t.t'
        email_confirmed = 'confirmed@t.t'

        self.user.add_unconfirmed_email(email_unconfirmed)
        self.user.add_confirmed_email(email_confirmed)

        self.assertIn(email_confirmed, self.user.get_confirmed_emails())
        self.assertIn(email_unconfirmed, self.user.get_unconfirmed_emails())

        # can't remove the primary
        with self.assertRaises(EmailIsPrimary):
            self.user.remove_email(self.user.email)

        # can remove the unconfirmed
        self.user.remove_email(email_unconfirmed)
        self.assertNotIn(email_unconfirmed, self.user.get_unconfirmed_emails())

        # can remove the confirmed
        self.user.remove_email(email_confirmed)
        self.assertNotIn(email_confirmed, self.user.get_confirmed_emails())


class PrimaryEmailTestCase(TestCase):

    def setUp(self):
        email = 'nobody@important.com'
        self.user = get_user_model().objects.create_user('uname', email=email)

    def test_set_primary_email(self):
        "Set an email to priamry"
        # set up two emails, confirm them post
        email1 = '1@t.t'
        self.user.add_confirmed_email(email1)
        self.user.set_primary_email(email1)

        email2 = '2@t.t'
        self.user.add_confirmed_email(email2)

        # assert signal fires as expected
        def listener(sender, **kwargs):
            self.assertEqual(sender, self.user)
            self.assertEqual(kwargs.get('old_email'), email1)
            self.assertEqual(kwargs.get('new_email'), email2)
        primary_email_changed.connect(listener)

        self.user.set_primary_email(email2)

        self.assertEqual(self.user.get_primary_email(), email2)

    def test_attempt_set_primary_email_to_unowned_email(self):
        "Try to set a primary email to one that doesn't belong to the user"
        other_user = get_user_model().objects.create_user(
            'myname', email='somebody@important.com',
        )
        email = '1@t.t'
        other_user.add_confirmed_email(email)

        with self.assertRaises(EmailNotConfirmed):
            self.user.set_primary_email(email)

    def test_attempt_set_primary_email_to_unconfirmed_email(self):
        "Try to set a primary email to one that hasn't been confirmed"
        email = '1@t.t'
        self.user.add_unconfirmed_email(email)

        with self.assertRaises(EmailNotConfirmed):
            self.user.set_primary_email(email)

    def test_getting_primary_email_with_mixin(self):
        "Try to get the primary email of a user model with the mixin"
        email = get_user_primary_email(self.user)
        self.assertEqual(email, self.user.email)

    def test_getting_primary_email_without_mixin(self):
        "Try to get the primary email of a user model without the mixin"
        model = apps.get_model('myapp', 'UserWithoutMixin')
        other_user = model.objects.create(email='somebody@important.com')
        email = get_user_primary_email(other_user)
        self.assertEqual(email, other_user.email)


class AddEmailIfNotExistsTestCase(TestCase):

    def setUp(self):
        self.email1 = 'e1@go.com'
        self.email2 = 'e2@go.com'
        self.email3 = 'e3@go.com'
        self.email4 = 'e4@go.com'

        # adds this email as an unconfirmed email
        self.user = get_user_model().objects.create_user(
            'uname', email=self.email1
        )

    def test_add_new_unconfirmed_email(self):
        result = self.user.add_email_if_not_exists(self.email2)

        self.assertEqual(self.user.email_address_set.count(), 2)
        address = self.user.email_address_set.get(key=result)
        self.assertEqual(address.email, self.email2)
        self.assertEqual(address.is_confirmed, False)

    def test_add_old_unconfirmed_email(self):
        self.user.add_unconfirmed_email(self.email2)
        self.user.add_unconfirmed_email(self.email3)

        address = self.user.email_address_set.get(email=self.email2)
        org_key, org_at = address.key, address.set_at

        sleep(0.1)
        result = self.user.add_email_if_not_exists(self.email2)

        self.assertEqual(self.user.email_address_set.count(), 3)
        address = self.user.email_address_set.get(key=result)
        self.assertEqual(address.email, self.email2)
        self.assertEqual(address.is_confirmed, False)
        self.assertNotEqual(address.key, org_key)
        self.assertGreater(address.set_at, org_at)

    def test_add_confirmed_email(self):
        self.user.add_confirmed_email(self.email2)
        self.user.add_confirmed_email(self.email3)

        result = self.user.add_email_if_not_exists(self.email2)

        self.assertIsNone(result)
        self.assertEqual(self.user.email_address_set.count(), 3)
        address = self.user.email_address_set.get(email=self.email2)
        self.assertEqual(address.is_confirmed, True)


class AutoAddTestCase(TestCase):

    def setUp(self):
        self.email = 'nobody@important.com'

    def test_email_is_added_if_not_empty(self):
        user = get_user_model().objects.create_user(
            'uname', email=self.email
        )
        self.assertEqual(user.email_address_set.count(), 1)

        email_address = user.email_address_set.first()
        self.assertFalse(email_address.is_confirmed)
        self.assertEqual(email_address.email, self.email)

    def test_email_is_not_added_if_empty(self):
        user = get_user_model().objects.create_user(
            'uname'
        )
        self.assertEqual(user.email_address_set.count(), 0)
        self.assertFalse(user.is_confirmed)



"Simple Email Confirmation Exceptions"


class SimpleEmailConfirmationException(Exception):
    pass


class EmailNotConfirmed(SimpleEmailConfirmationException):
    pass


class EmailConfirmationExpired(SimpleEmailConfirmationException):
    pass


class EmailIsPrimary(SimpleEmailConfirmationException):
    pass



__version__ = '0.4'
__all__ = [
    'email_confirmed',
    'unconfirmed_email_created',
    'primary_email_changed',
]


from .signals import (
    email_confirmed, unconfirmed_email_created, primary_email_changed,
)



from django.contrib import admin

from .models import EmailAddress


class EmailAddressAdmin(admin.ModelAdmin):
    list_display = ('user', 'email', 'key', 'set_at', 'confirmed_at')
    search_fields = ('email', 'key')

admin.site.register((EmailAddress,), EmailAddressAdmin)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import django.utils.timezone
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='EmailAddress',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('email', models.EmailField(max_length=255)),
                ('key', models.CharField(unique=True, max_length=40)),
                ('set_at', models.DateTimeField(default=django.utils.timezone.now, help_text='When the confirmation key expiration was set')),
                ('confirmed_at', models.DateTimeField(help_text='First time this email was confirmed', null=True, blank=True)),
                ('user', models.ForeignKey(related_name='email_address_set', to=settings.AUTH_USER_MODEL, on_delete=models.CASCADE)),
            ],
            options={
                'verbose_name_plural': 'email addresses',
            },
            bases=(models.Model,),
        ),
        migrations.AlterUniqueTogether(
            name='emailaddress',
            unique_together=set([('user', 'email')]),
        ),
    ]



from django.dispatch import Signal

email_confirmed = Signal(providing_args=['user', 'email'])
unconfirmed_email_created = Signal(providing_args=['user', 'email'])
primary_email_changed = Signal(
    providing_args=['user', 'new_email', 'old_email'],
)



import re
import sys
from os import path
from setuptools import setup

# read() and find_version() taken from jezdez's python apps, ex:
# https://github.com/jezdez/django_compressor/blob/develop/setup.py

def read(*parts):
    return open(path.join(path.dirname(__file__), *parts)).read()


def find_version(*file_paths):
    version_file = read(*file_paths)
    version_match = re.search(
        r"^__version__ = ['\"]([^'\"]*)['\"]", version_file, re.M,
    )
    if version_match:
        return version_match.group(1)
    raise RuntimeError("Unable to find version string.")


setup(
    name='django-simple-email-confirmation',
    version=find_version('simple_email_confirmation', '__init__.py'),
    author='Mike Fogel',
    author_email='mike@fogel.ca',
    description="Simple email confirmation for django.",
    long_description=read('README.rst'),
    url='https://github.com/mfogel/django-simple-email-confirmation',
    license='BSD',
    packages=[
        'simple_email_confirmation',
        'simple_email_confirmation.migrations',
        'simple_email_confirmation.south_migrations',
        'simple_email_confirmation.tests',
        'simple_email_confirmation.tests.myproject',
        'simple_email_confirmation.tests.myproject.myapp',
    ],
    install_requires=['django>=1.7.0' if sys.version_info[0] > 2
        else 'Django<2.0'],
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        'License :: OSI Approved :: BSD License',
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        'Topic :: Utilities',
        "Framework :: Django",
    ]
)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

from django.contrib.auth import get_user_model
User = get_user_model()
user_orm_label = '%s.%s' % (User._meta.app_label, User._meta.object_name)
user_model_label = '%s.%s' % (User._meta.app_label, User._meta.module_name)


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'EmailAddress'
        db.create_table('simple_email_confirmation_emailaddress', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('user', self.gf('django.db.models.fields.related.ForeignKey')(related_name='email_address_set', to=orm[user_orm_label])),
            ('email', self.gf('django.db.models.fields.EmailField')(max_length=255)),
            ('key', self.gf('django.db.models.fields.CharField')(unique=True, max_length=40)),
            ('set_at', self.gf('django.db.models.fields.DateTimeField')()),
            ('confirmed_at', self.gf('django.db.models.fields.DateTimeField')(null=True, blank=True)),
        ))
        db.send_create_signal('simple_email_confirmation', ['EmailAddress'])

        # Adding unique constraint on 'EmailAddress', fields ['user', 'email']
        db.create_unique('simple_email_confirmation_emailaddress', ['user_id', 'email'])


    def backwards(self, orm):
        # Removing unique constraint on 'EmailAddress', fields ['user', 'email']
        db.delete_unique('simple_email_confirmation_emailaddress', ['user_id', 'email'])

        # Deleting model 'EmailAddress'
        db.delete_table('simple_email_confirmation_emailaddress')


    models = {
        'simple_email_confirmation.emailaddress': {
            'Meta': {'unique_together': "(('user', 'email'),)", 'object_name': 'EmailAddress'},
            'confirmed_at': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'key': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '40'}),
            'set_at': ('django.db.models.fields.DateTimeField', [], {}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'email_address_set'", 'to': "orm['%s']" % user_orm_label, 'on_delete': models.CASCADE})
        },
        user_model_label: {
        },
    }

    complete_apps = ['simple_email_confirmation']
